# -*- coding: utf-8 -*-

from django.template import Template, Context
from django.test import TestCase

from bs4 import BeautifulSoup

from real_content.drc_utils import (
    get_language,
    get_text_file,
    save_content,
    parse_url
    )
from real_content.settings import (
    DRC_MISSING_FILE_MSG,
    DRC_EMPTY_FILE_MSG,
    DRC_LANGUAGE
    )


class TitleTagTest(TestCase):
    def test_title_shows_up(self):
        template = Template('{% load drc %} {% drc_title 2 %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        h2_tag = soup.find('h2')

        self.assertTrue(h2_tag, 'title tag missing')
        self.assertFalse(DRC_MISSING_FILE_MSG in str(h2_tag),
            'language file missing')
        self.assertFalse(DRC_EMPTY_FILE_MSG in str(h2_tag),
            'language file empty')
        self.assertFalse(h2_tag.is_empty_element, 'title tag is empty')

    def test_title_multiple_words_in_param(self):
        template = Template('{% load drc %} {% drc_title "h3 data-arg" %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        h3_tag = soup.find('h3')

        self.assertFalse(h3_tag.has_attr('data-arg'))

    def test_title_empty_param(self):
        template = Template('{% load drc %} {% drc_title "" %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        h1_tag = soup.find('h1')

        self.assertTrue(h1_tag, 'title tag missing')

    def test_title_h_only_number_bigger_than_6(self):
        template = Template('{% load drc %} {% drc_title 9 %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        h6_tag = soup.find('h6')

        self.assertTrue(h6_tag, 'title tag missing')


class ParagraphsTagTest(TestCase):

    def test_paragraphs_show_up(self):
        template = Template('{% load drc %} {% drc_paragraphs 2 %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        p_tags = soup.find_all('p')

        self.assertTrue(p_tags, 'paragraph tags missing')
        self.assertTrue(len(p_tags) == 2, 'not enough paragraphs')
        self.assertFalse(DRC_MISSING_FILE_MSG in str(p_tags[0]),
            'language file missing')
        self.assertFalse(DRC_EMPTY_FILE_MSG in str(p_tags[0]),
            'language file empty')
        self.assertFalse(p_tags[0].is_empty_element, 'paragraph tag is empty')

    def test_paragraphs_only_one_p(self):
        template = Template('{% load drc %} {% drc_paragraphs 1 %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        p_tags = soup.find_all('p')

        self.assertTrue(p_tags, 'paragraph tags missing')
        self.assertTrue(len(p_tags) == 1, 'not enough paragraphs')


class ImageTagTest(TestCase):
    def test_image_shows_up(self):
        template = Template('{% load drc %} {% drc_image 300 200 "sport" %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        img_tag = soup.find('img')

        self.assertTrue(img_tag, 'image tag missing')
        img_source = img_tag.attrs.get('src')
        self.assertTrue('300/200' in img_source, 'wrong image dimensions')
        self.assertTrue('300/200/sport' in img_source, 'wrong image category')

    def test_image_gray(self):
        template = Template('{% load drc %} {% drc_image 300 200 gray=True %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        img_tag = soup.find('img')

        img_source = img_tag.attrs.get('src')
        self.assertTrue('/g/' in img_source, 'image not in grayscale')

    def test_image_specific_id(self):
        template = Template(
            '{% load drc %} {% drc_image category="people" image_id=2 %}')
        rendered = template.render(Context({}))
        soup = BeautifulSoup(rendered, 'html.parser')
        img_tag = soup.find('img')

        img_source = img_tag.attrs.get('src')
        self.assertTrue('/people/2' in img_source, 'specific image id not set')


class NumberTagTest(TestCase):
    def test_number_shows_up(self):
        template = Template('{% load drc %} {% drc_number 1 100 %}')
        rendered = template.render(Context({}))
        number = int(rendered)
        self.assertTrue(1 <= number <= 100, 'number out of bounds')

    def test_number_value_error(self):
        template = Template('{% load drc %} {% drc_number 1 "nope" %}')
        rendered = template.render(Context({}))

        self.assertEqual(rendered.strip(),
            'please provide a number as an argument',
            'argument of wrong type passed')

    def test_number_when_2nd_number_bigger_than_1st(self):
        template = Template('{% load drc %} {% drc_number 100 1 %}')
        rendered = template.render(Context({}))
        number = int(rendered)
        self.assertTrue(1 <= number <= 100, 'number out of bound')


class GetLanguage(TestCase):
    def test_get_language(self):
        self.assertEqual(DRC_LANGUAGE, get_language())
        self.assertNotEqual('hr', get_language())
        self.assertEqual('hr', get_language('hr'))


class GetTextFileTest(TestCase):
    def test_existing_file(self):
        text_file = get_text_file('en')
        self.assertIsNotNone(text_file)
        self.assertTrue(hasattr(text_file, 'read'))

    def test_nonexisting_file(self):
        text_file = get_text_file('xyz')
        self.assertIsNone(text_file)


class SaveContentTest(TestCase):
    def setUp(self):
        content = (
                '<!DOCTYPE html>'
                '<html>'
                '<body>'
                '<h1>My First Heading</h1>'
                '<p>My first paragraph.</p>'
                '<p>My second paragraph.</p>'
                '<h2>My Second Heading</h2>'
                '<p>My third paragraph.</p>'
                '</body>'
                '</html>'
            )
        self.soup = BeautifulSoup(content, 'html.parser')
        self.lang = 'test_lang'

        # deleting the old file from previous tests
        import os
        content_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), 'content'))
        content_file_path = os.path.join(
            content_dir, '{}_{}.txt'.format(self.lang, 'paragraphs'))
        os.remove(content_file_path)

    def test_save(self):
        save_content(self.soup, 'p', text_type='paragraphs',
            url='www.example.com', language=self.lang, min_length=10)

        text_file = get_text_file(self.lang, text_type='paragraphs')
        self.assertIsNotNone(text_file)
        self.assertTrue(hasattr(text_file, 'read'))

        lines = text_file.read().splitlines()
        clean_lines = [line for line in lines
            if line.startswith('#') is False and line.strip() != '']
        self.assertEqual(len(clean_lines), 3)

        self.assertTrue(any(line for line in lines
            if line.startswith('# www.example.com')))


class ParseUrl(TestCase):
    def setUp(self):
        self.real_url = 'https://www.yahoo.com'
        self.fake_url = 'http://www.jibber-jabber-asdasdasd-asffgdsgg.com'
        self.lang = 'test_lang'

    def test_parse_url(self):
        result = parse_url(self.real_url, self.lang)
        self.assertEqual(result, 'success!')

        result = parse_url(self.fake_url, self.lang)
        self.assertEqual(result, 'error getting content from url')



# -*- coding: utf-8 -*-

from django.conf import settings

__all__ = [
    'DRC_LANGUAGE',
    'DRC_MISSING_FILE_MSG',
    'DRC_EMPTY_FILE_MSG',
    'DRC_NUMBER_START',
    'DRC_NUMBER_END',
]

# setting english as the default language
DRC_LANGUAGE = getattr(settings, 'DRC_LANGUAGE', 'en')
DRC_MISSING_FILE_MSG = 'language file does not exists'
DRC_EMPTY_FILE_MSG = 'no usable lines in language file'
# random number tag limits
DRC_NUMBER_START = getattr(settings, 'DRC_NUMBER_START', 1)
DRC_NUMBER_END = getattr(settings, 'DRC_NUMBER_END', 1000)



# -*- coding: utf-8 -*-

import random

from django import template
from django.utils import six

from real_content.drc_utils import get_language, get_text_file
from real_content.settings import (
    DRC_MISSING_FILE_MSG,
    DRC_EMPTY_FILE_MSG,
    DRC_NUMBER_START,
    DRC_NUMBER_END
    )

register = template.Library()


def random_lines(text_file, no_of_lines=1):
    u"""
    Gets a random line from a file ignoring lines starting with # and empty
    lines.
    """
    if text_file is None:
        return DRC_MISSING_FILE_MSG
    lines = text_file.read().splitlines()
    clean_lines = [line for line in lines
        if line.startswith('#') is False and line.strip() != '']

    if clean_lines == []:
        return DRC_EMPTY_FILE_MSG
    if no_of_lines == 1:
        return random.choice(clean_lines)
    else:
        return random.sample(clean_lines, no_of_lines)


@register.inclusion_tag('real_content/tags/drc_title.html')
def drc_title(tag='h1', css_class='', language=''):
    u"""
    Retrieve random title.
    Uses langauge from global settings if an override is not provided.
    """
    language = get_language(language)
    titles = get_text_file(language, 'titles')

    data = {
        'title': random_lines(titles, 1),
    }

    # backwards compatibility with 0.1.5 - if only heading size was sent
    try:
        tag = int(tag)
        if tag > 6:
            tag = 6
        tag = 'h{}'.format(tag)
    except ValueError:
        pass

    # make sure that only/at least one word is the tag
    # (important for closing tags)
    try:
        data['tag'] = tag.split()[0]
    except IndexError:
        data['tag'] = 'h1'

    data['css_class'] = css_class
    return data


@register.inclusion_tag('real_content/tags/drc_paragraph.html')
def drc_paragraphs(no_of_paragraphs=1, css_class='', language=''):
    u"""
    Retrieve n random paragraphs.
    Uses langauge from global settings if an override is not provided.
    """
    language = get_language(language)
    paragraphs = get_text_file(language, 'paragraphs')

    random_paragraphs = random_lines(paragraphs, no_of_paragraphs)

    # fix when returning only 1 paragraph
    if isinstance(random_paragraphs, six.string_types):
        temp_list = []
        temp_list.append(random_paragraphs)
        random_paragraphs = temp_list

    data = {
        'paragraphs': random_paragraphs,
    }

    data['css_class'] = css_class
    return data


@register.inclusion_tag('real_content/tags/drc_lorempixel.html')
def drc_image(width=640, height=480, category='',
    gray=False, image_id='', css_class=''):
    u"""
    Retrieve image from lorempixel.com service.
    Possible categories are:
        abstract, animals, business, cats, city, food, nightlife, fashion,
        people, nature, sports, technics, transport
    """
    data = {}
    url = u'http://lorempixel.com/'
    if gray:
        url = '{url}g/'.format(url=url)
    url = '{url}{width}/{height}/'.format(url=url, width=width, height=height)
    if category:
        url = '{url}{category}/'.format(url=url, category=category)

        # image_id can only be defined if the category was set, otherwise we
        # get black image
        if image_id and image_id <= 10:
            url = '{url}{image_id}'.format(url=url, image_id=image_id)
        else:
            # use random image_id if non was set to prevent from displaying the
            # same image over and over again
            url = '{url}{image_id}'.format(
                url=url,
                image_id=random.randrange(1, 11))

    data['url'] = url
    data['css_class'] = css_class
    return data


@register.simple_tag
def drc_number(start=DRC_NUMBER_START, end=DRC_NUMBER_END):
    u"""
    Return a random number inbetween start and end limits.
    """
    try:
        int(start), int(end)
        if start <= end:
            return random.randint(int(start), int(end))
        else:
            return drc_number(end, start)
    except ValueError:
        return 'please provide a number as an argument'
